/*
 * Copyright (c) 1993/08  Kenji Arisawa. All right reserved.
 * License to freely use and distribute this software is hereby granted 
 * by the author, subject to the condition that this copyright notice 
 * remains intact.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY WARRANTIES.
 *
 * Half of codes in this file is due to
 *	  Doug Kingston	<dpk@morgan.com>
 *	  Morgan Stanley & Co. (19th Floor)
 *	  1251 Avenue of the Americas
 *	  NY, NY, 10020
 *	  (212) 703-7541
 *
 * I showed my coding by ( tip+ )
 * on the otherhand Kingston's by ( tip2 ).
 * Great deal of reconstruction have been done for tip+ support
 * and much of tip2 codings have been discarded.
 * However I owe my new codings largely to his preceding work.
 * -Kenar-
 */

/*
 *	login.c
 *
 *	History:
 *  1989/4  Implemented to tip for slip support by Doug Kingston
 *	1993/8	Reconstructed and enriched by Kenar
 *	1995/8	Bugfix in sort routine by Kenar
 *	1995/12	Changed usage of login by Kenar
 *  1996/01 Added fail argument and some special symbols
 */
 #ifndef lint
static char sccsid[] = "@(#)login.c	1.0 (Kingston/Kenar) 2/3/96";
#endif /* not lint */

#include <sys/time.h>
#include "tip.h"

extern int suidmode;

typedef struct
{	int lineNo;
	char *cmdLn;
	int argc;
	char **argv;
}	ScriptLineType;

#define MAXLINE 512
#define LINESIZ 128

static char* execScript();
static char *dosend();
static sigfunc_t naptimeout();
static char *scriptErrMsg();
static char *dowait();
static void echo(char*);
static int getch();
static int waitStrs();
static int dogoto();
static void strfix(char* cp);
static char octalnum(char **cpp);

static char _got[80];
static ScriptLineType *_wp; /* the last wait line */
static char *script;
static int vmode; /* visible mode controled by "+v" & "-v" in Host.tip+ */


/*
 * keyint: keyboard interrupt trap. ( tip+ )
 */

static jmp_buf jmpint;

static sigfunc_t keyint()
{
	longjmp(jmpint,1);
}


/*
 * login: execute login script. ( tip+ )
 * original login() is discarded by Kenar.
 * -Kenar-
 */
char *login(path)
char *path;
/* return:
 *    NULL if no errors
 *    ""   if file not found
 *    "-"  fail without error message
 *    else
 *		error message in scripting:
 *      1. the message in fail command if control reached fail
 *      or
 *      2.  syntax error message if failed to reach "fail".
 */
{	char buf[LINESIZ+2];
	int n,k,i,line;
	ScriptLineType sLine[MAXLINE];
	char *tmpArg[LINESIZ/2];
	char *p,*q;
	FILE *fp;
	sigfunc_t (*intsave)();

	vmode = NO;
	/*
	 * reading tip variables from the path
	 * -Kenar-
	 */
	fp = fopen(path,"r");
	if(fp){ /* Read parameters from fp */
		printf("[script: %s]\r\n",path);
		script = path;
		line = vinitialize(fp,"begin");
	}
	else return "";

	vcheck();

	/*
	 * Read script file and store the data to memory.
	 * Coded by,
	 * -Kenar-
	 */
	n = 0;    /* n is the effective script line number */
	while(fgets(buf, sizeof(buf), fp)){
		p = trim(buf);
		if(equal(p,"end")) break;
		line++;
		if(!*p) continue; /* blank line */
		if (*p == '#') continue; /* comment line */
		
		/* Don,t label conversion move here.
		   Because we have a case such like:
		   wait 5/error login:
		*/
		
		sLine[n].lineNo = line;
		sLine[n].cmdLn = makstr(p); /* we keep cmdLn */
		q = makstr(p); /* q is to be cooked */
		
		if(oneof(arg0(p),"echo,fail"))
		{	int linesiz,cmdsiz;
			linesiz = strlen(q);
			cmdsiz = strlen(arg0(p));
			sLine[n].argc = 2;
			sLine[n].argv = (char **)calloc(3,sizeof(char*));
			q[cmdsiz] = '\0';
			sLine[n].argv[0] = q;
			if(cmdsiz < linesiz)
				sLine[n].argv[1] = q + cmdsiz + 1;
			else
				sLine[n].argv[1] = "";
			sLine[n].argv[2] = NULL;
			/* EXAMPLE:
			 * The command line
			 * 1. "echo abc" -> argv[0]="echo", argv[1]="abc"
			 * 2. "echo"     -> argv[0]="echo", argv[1]=""
			 */
		}
		else
		{	sLine[n].argc = k = 
				sepstr(q,tmpArg," \t");
			sLine[n].argv = (char**)calloc(k + 1,sizeof(char*));
			for(i = 0; i < k ; i++)
				sLine[n].argv[i] = tmpArg[i];
			sLine[n].argv[k] = NULL;

			/* convert "LABEL:" to "label LABEL"  */
			p = sLine[n].argv[0];
			if(p[(k = strlen(p)-1)] == ':')
			{	p[k] = '\0';
				sLine[n].argv =
					(char**)calloc(3,sizeof(char*));
				sLine[n].argv[1] = p;
				sLine[n].argv[0] = "label";
				sLine[n].argv[2] = NULL;
				sLine[n].argc = 2;
			}
		}		
		n++;
	}

	if(suidmode)
		p = execScript(sLine,n); /* n is line count */
	else{
		if(!setjmp(jmpint)){
			intsave = signal(SIGINT,keyint);
			p = execScript(sLine,n); /* n is line count */
		}
		else{
			puts("User termination and continue..");
			p = NULL;
		}
		signal(SIGINT,intsave);
	}
	fclose(fp);
	if(p && equal(p,"")) p = "-";
	return p;
}



/*
 * vinitialize: initialize tip variables in script file. ( tip+ )
 * coded by Kenar
 */
int vinitialize(fp,term)
FILE *fp;
char *term; /* terminater */
/* return: line count
 */
{	char buf[256];
	char *tp;
	int n;
	
	n = 0;
	while (fgets(buf, sizeof(buf), fp) != NULL)
	{	n++;
		tp = trim(buf);
		if (term && equal(tp,term)) break;
		if(*tp == '#') continue; /* skip comment line */
		if(*tp)
		{	if (vflag) printf("set %s\n", tp);
			vlex(tp);
		}
	}
	return n;
}

/*
 * mkmsg: make string for message that is used in echo() and log()
 * coded by Kenar
 * -Kenar-
 */
static char *mkmsg(bp,s)
	char *bp,*s;
{	char c;
	char *t;
	t = bp;
	*t = 0;
	while(*s){
		c = *s++;
		switch(c){
		case '@':
			t = strapp(t,_got);
			break;
		case '^':
			sprintf(t,"%d",_wp->lineNo);
			t = index(t,'\0');
			break;
		case '_':
			t = strapp(t,_wp->cmdLn);
			break;
		case '\\':
			switch(*s){
			case '@':
			case '^':
			case '_':
			case '\\':
				*t++ = *s;
				break;
			default:
				*t++ = c;
				*t++ = *s;
				break;
			}
			s++;
			break;
		default:
			*t++ = c;
			break;
		}
	}
	*t = '\0';
	return bp;
}


/*
 * echo: echo in direction to tip+ script. ( tip+ ).
 * coded by Kenar
 */
static void echo(s)
char *s;
{	char buf[STRSIZ];
	s = mkmsg(buf,s);
	puts(s);
}


static char *failErrMsg(s)
char *s;
{	static char buf[STRSIZ];
	s = mkmsg(buf,s);
	return s; /* Never return NULL */
}


/*
 * execScript: execute script line. ( tip+ )
 * Return value:
 *   NULL :  if no error
 *   ""   :  if control reached "fail" and no argument in "fail".
 *   error message : That is written in "fail" line,
 *                   or syntax error message if failed to reach "fail".
 * coded by Kenar
 */
static char* execScript(sLine,lineCount)
ScriptLineType *sLine;
int lineCount;
{	int n,line;
	int cnt;
	char *cmd,*cp,**q;
	ScriptLineType *p;

	for(n = 0; n < lineCount; n++)
	{	p = &sLine[n];
		cmd = p->argv[0];
		cnt = p->argc;
		line = p->lineNo;
		if(vflag){	int k;
			printf("%3d:",line);
			for(k = 0; p->argv[k]; k++) printf(" %s",p->argv[k]);
			putchar('\n');
		}
		if (equal(cmd, "done")){
			if(vmode) putc('\n',stderr);
			return(NULL);
		}
		else if (equal(cmd, "fail")) return(failErrMsg(p->argv[1]));
		else if(equal(cmd, "send"))
		{	if(cnt != 2) return(scriptErrMsg(line));
			if (cp = dosend(p->argv[1])) return(cp);
		}
		else if(equal(cmd, "wait"))
		{	q = p->argv;
			cp = dowait(++q);
					/* label name is returned */
			_wp = p;
			if(!cp) return(scriptErrMsg(line));
			if(!equal(cp,""))
			{	n = dogoto(cp,sLine,lineCount);
				if(n < 0)
				{	fprintf(stderr,
						"label %s not found\n",cp);
					return(scriptErrMsg(line));
				}
			}
		}
		else if(equal(cmd, "label")) continue;
		else if(equal(cmd, "goto"))
		{	if(cnt != 2) return(scriptErrMsg(line));
			n = dogoto(p->argv[1],sLine,lineCount);
			if(n < 0) return(scriptErrMsg(line));
		}
		else if(equal(cmd,"echo")) echo(p->argv[1]);
		else if(equal(cmd,"+v"))
		{	vmode = YES;
			continue;
		}
		else if(equal(cmd,"-v"))
		{	vmode = NO;
			continue;
		}
		else
			return(scriptErrMsg(line));
	}
	if (vflag) putc('\n', stderr);
	return(NULL);
}

/*
 * scriptErrMsg: put syntax error in tip+ script. ( tip+ )
 * coded by Kenar.
 */
static char* scriptErrMsg(line) int line;
{	static char errmsg[80];
	sprintf(errmsg, "Error in %.80s at line %d", script,line);
	return(errmsg);
}

/*
 * dosend: send a string processing some special symbols. ( tip2 )
 */
static char *dosend(cp)
char *cp;
{
	char c;
	char *pn = NULL;

	if(vflag)
		fprintf(stderr, "send: %s: ", cp);
	while(*cp) {
		c = *cp++;
		if (c == '\\') {
			switch(*cp) {
			case 0:
				return("send format error");
			case 'b':
				c = '\b'; cp++; break;
			case 'd':
				sleep(1); cp++;
				continue;
			case 'f':
				c = '\f'; cp++; break;
			case 'n':
				c = '\n'; cp++; break;
			case 'r':
				c = '\r'; cp++; break;
			case 's':
				c = ' '; cp++; break;
			case 't':
				c = '\t'; cp++; break;
			case 'x':
				genbrk(); cp++;
				continue;
			case '?':
				c = 0177; cp++; break;
			case '#':
				pn = PN; cp++; break;
			default:
				c = octalnum(&cp);
			}
		}
		if(pn){
			do{
				if (pwrite(pn, 1) != 1)
					return("write error to remote site");
				if (vflag)
					show(pn, 1);
				nap(20);
				pn++;
			} while(*pn);
			pn = NULL;
		}
		else{
			if (pwrite(&c, 1) != 1)
				return("write error to remote site");
			if (vflag)
				show(&c, 1);
			if(c == '\r') nap(nvalue(LDELAY));
			else nap(20);
		}
	}
	if (vflag)
		putc('\n', stderr);
	return(0);
	
}

/*
 * nap: sleep a little ( tip2 )
 * EXAMPLE
 * nap(20) sleeps 20ms
 * nap(0)  clears timer. ( 1996/01 tip+ )
 * -Kenar-
 */
void nap(ms)
int ms;
{
	struct itimerval itv;
	signal(SIGALRM, naptimeout);
	itv.it_interval.tv_sec = 0;
	itv.it_interval.tv_usec = 0;
	itv.it_value.tv_sec = 0;
	itv.it_value.tv_usec = ms*1000;
	setitimer(ITIMER_REAL, &itv, (struct itimerval *)0);
	/* The ITIMER_REAL timer decrements in real time.
	 * A SIGALRM signal is delivered when this timer expires.
	 * FROM man setitimer
	 * -Kenar- */
	if(ms > 0){
		sigpause(0);
		/* Sigpause always terminates by being interrupted,
		 * returning -1 with errno set to EINTR.
		 * FROM man sigpause
		 * Kenar say,
		 *   I will use SIGALRM for this purpose. This works well.
		 * -Kenar- */
		nap(0);
	}
}

static sigfunc_t naptimeout()
{
	/* make SIGALRM no effect */
	/* return(0); */
}

static jmp_buf jmpbuf;
static sigfunc_t waittimeout();

/*
 * dowait: wait strings from modem under the direction of tip+ script.
 * rewritten by Kenar for tip+ support.
 */
static char *dowait(argv)	/* returns label name */
char **argv;
/*	dowait returns label name.
	if no label then returns NULL string (""); (no label is also legal syntax)
	if error then returns NULL */
{
	lett *strs[128],*labs[128];
	lett *label,*s;
	lett *control = argv[0];
	int secs;
	int j,n,m;

	if(*control == '*') secs = nvalue(DIALTIMEOUT);
	else secs = atoi(control);
	if (label = index(control, '/'))
		label++;
	if (setjmp(jmpbuf)) {
		alarm(0);
		if(vflag) fprintf(stderr,
				"\nwait: timeout, going to %s\n", label);
		strcpy(_got,"TIME OUT");
		return(label);
	}
	signal(SIGALRM, waittimeout);
	alarm(0);
	
	argv++;
	n = 0;
	while(*argv)
	{	m = n;
		while(*argv && **argv != '/')
		{	strs[n] = *argv;
			n++; argv++;
		}
		
		if(m == n) /* syntax err: no string to wait */
			return NULL;
		if(*argv) /* then *argv is label */
		{	s = *argv;
			s++;
		}
		else	/* no label */
			s = "";
		
		while(m < n)
		{	labs[m] = s;
			m++;
		}
		
		if(!*argv) break;

		argv++;
	}
	
	if(vflag)
	{	fprintf(stderr, (char*)
			(n == 1 ? 	"Waiting %d string ... ":
			/* else */	"Waiting %d strings ... "), n);
		for(j = 0; j < n; j++)
		{	fputs(strs[j],stderr);
			putc(' ',stderr);
		}
		putc('\n',stderr);
	}
	
	for(j = 0; j < n; j++) strfix(strs[j]);
	
	/*	------------ SORT ROUTINE -----------------
	 * 	strs should be sorted
	 *	before handed over to waitStrs
	 *	STRAIGHT SELECTION
	 *	ref: Wirth,"Algorithm and Data Structure"
	 *	external variable: n,strs,labs
	 *	coded by Kenar
	 */
	{	int i,j,k;
		lett *x,*t;

		for(i = 0; i < n - 1; i++){
			k = i;
			for(j = i + 1; j < n; j++)
				if(strcmp(strs[j],strs[k]) < 0) k = j;
			/* swap i and k */
			x = strs[i]; strs[i] = strs[k]; strs[k] = x;
			t = labs[i]; labs[i] = labs[k]; labs[k] = t;
		}

	}

	alarm(secs);
	n = waitStrs(n,strs);
	alarm(0);
	if(n == ERR) return NULL;
	if(vflag) fprintf(stderr,"\nReceived: %s\n",strs[n]);
	strcpy(_got,strs[n]);
	return(labs[n]);
}


static int _ungetcount;
static lett _ungetbuf[512];

/*
 * getch: read a char from modem. ( tip+ )
 * coded by Kenar
 */
static int getch()
{	lett buf[1];
	if(_ungetcount > 0)
	{	_ungetcount--;
		return _ungetbuf[_ungetcount];
	}
	else if(pread(buf,1) < 1)
	{	fputs("Error in reading device\n",stderr);
		return ERR;
	}
	else
	{	if(vflag) show(buf,1);
		else if(vmode) putc(buf[0],stderr);
		return buf[0];
	}
}

/*
 * ungetch: unget n chars. ( tip+ )
 * codey by Kenar
 */
static void ungetch(s,n)
lett *s;
int n;
{	while(n > 0)
	{	n--;
		_ungetbuf[_ungetcount] = s[n];
		_ungetcount++;
	}
}

/*
 * waitStrs: wait n strings  ( tip+ )
 * coded by Kenar
 */
static int waitStrs(n,strs)
int n;  /* count of string */
lett **strs; /* string pointer */
{	int j,k;

	int c,m;

	lett buf[512];
	int lenStr[128];
	
	if(n == 0) for(;;) c = getch(); /* wait forever discarding data. */

	for(j = 0; j < n; j++) lenStr[j] = strlen(strs[j]);
	

	/*	<<Search strings in a stream>>		 *		---- coded by Kenar ----
	 *
	 *	Target strings must be sorted.		 *
	 *	Be careful with the following example: 	 *		stream : AAAAB
	 *		target : AAAB
	 *	This example show:
	 *		when the stream	data begin
	 *		matching with target, they
	 *		should be stocked in stack,
	 *		and when difference is found	 *		comparison should be started 	 *		from second character in the 	 *		stack.
	 *	In this codes the stack is named buf.
	 *
	 *  -------------------- CAUTION ----------------------------
	 *  The codes bellow are the engine of tip+ script.
	 *  Very complicated to understand and therefore never touch!
	 *  ----------------------------------------------------------
	 *  -Kenar-
	 */
	for(;;)
	{	m = 0; j = 0; k = n - 1;
		while(j < k)
		{	c = getch();
			if(c == ERR) break;
			while (j < n && c > strs[j][m]) j++;
			while (k >= 0 && c < strs[k][m]) k--;
			buf[m++] = c;
		}
		if ( j == k )
		{   while(m < lenStr[j])
			{	c = getch();
				if(c == ERR) break;
				buf[m] = c;
				if(strs[j][m] != c) break;
				m++;
			}
			if(m == lenStr[j]) return j; /* found */
			if(m > 1) ungetch(&buf[1],m - 1);
		}
		if(c == ERR) return ERR;
	}
}


/*
 * strfix: convert string in tip script to internal format. ( tip2 )
 */
static void strfix(cp) char *cp;
{
	char c;
	char *np = cp;

	while(*cp) {
		c = *cp++;
		if (c == '\\') {
			switch(*cp) {
			case 'b':
				c = '\b'; cp++; break;
			case 'f':
				c = '\f'; cp++; break;
			case 'n':
				c = '\n'; cp++; break;
			case 'r':
				c = '\r'; cp++; break;
			case 's':
				c = ' '; cp++; break;
			case 't':
				c = '\t'; cp++; break;
			case '?':
				c = 0177; cp++; break;
			default:
				c = octalnum(&cp);
			}
		}
		*np++ = c; 
	}
	*np = 0;
}

#define isoctal(c) ((c) >= '0' && (c) <= '7')

/*
 * octalnum: process octal number. ( tip2 )
 */
static char octalnum(cpp) char **cpp;
{
	int n = 0;
	char c = *((*cpp)++);

	if (isoctal(c)) {
		n = (c-'0');
		c = *((*cpp)++);
		if (isoctal(c)) {
			n = (n<<3)+(c-'0');
			c = *((*cpp)++);
			if (isoctal(c))
				n = (n<<3)+(c-'0');
		}
		c = n;
	}
	return(c);
}

/*
 * show: show memory of size n in printable format. ( tip2 )
 */
void show(cp, n)
char *cp;
int n;
{
	register int i;
	char c;

	for(i = 0; i < n; i++) {
		c = cp[i]&0177;
		if(isprint(c))
			fputc(c, stderr);
		else
			fprintf(stderr, "\\%03o", c);
	}
}

/*
 * waittimeout: wait time out. ( tip2 )
 */
static sigfunc_t waittimeout()
{
	longjmp(jmpbuf, 1);
}

/*
 * dogoto: go to label. ( tip+ )
 * original tip coding is replaced by Kenar
 * -Kenar-
 */
static int dogoto(label,sLine,lineCount)
char *label;
ScriptLineType *sLine;
int lineCount;
{	int n;

	if(vflag)
		fprintf(stderr, "goto: %s\n", label);
	for(n = 0; n < lineCount; n++)
	{	if(equal(sLine[n].argv[0], "label")
			&& (sLine[n].argc == 2)
			&& equal(label, sLine[n].argv[1]))
			return n;
	}
	return ERR;
}

