/*
 * Copyright (c) 1983 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
 * Copyright (c) 1995  Kenji Arisawa. All right reserved.
 * License to freely use and distribute this software is hereby granted 
 * by the author, subject to the condition that this copyright notice 
 * remains intact.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY WARRANTIES.
 */

/*
 * tip+ 2.1:
 * Great reduction of old codes using table operations.
 * -Kenar-
 */


#ifndef lint
static char sccsid[] = "@(#)remote.c	1.0 (Berkeley/Kenar) 1/3/96";
#endif /* not lint */

# include "tip.h"

extern int suidmode;
extern char **rdata;
char *rgetstr();

/*
 * Abreviations for STRING that is not tip variable,
 * but defined in remote.
 * -Kenar-
 */
static struct {
	char *cap;
	char **pval;
} divcap[] = {
	{"at", &AT},
	{"dv", &DV},
	{"cm", &CM},
	{"pn", &PN},
	{NULL, NULL}
};


/* negative boolean captions in remote */
static struct{
	char *cap;
	int  id;
} nbcap[] = {
	{"nt", TAND},
	{NULL, 0}
};

void getremcap(host) char *host;
{
	int stat;
	char tbuf[STRSIZ*4];
	static char buf[STRSIZ];
	char *bp = buf;
	char *r;
	int n,i;

	if (!suidmode && host[0] == '/' && access(host, R_OK | W_OK) == 0) {
		/* This case comes from: tip -9600 /dev/ttyfb */
		svalue(HOST) = DV = makstr(host);
		DU = NO;
		return;
	}
	if ((stat = getent(tbuf,host,svalue(REMOTE))) <= 0) {
		fprintf(stderr, stat == 0 ?
			"tip: unknown host %s\n" :
			"tip: can't open host description file\n", host);
		exit(3);
	}

	for (n = 0; divcap[n].cap != NULL; n++){
		r = rgetstr(divcap[n].cap, &bp);
		if(r) *divcap[n].pval = makstr(r);
	}

	n = 0;
	for (n = 0; vname(n) != NULL; n++)
		if(vrname(n))
		switch(vtype(n)&TMASK){
		case STRING:
			r = rgetstr(vrname(n), &bp);
			if(r) svalue(n) = makstr(r);
			break;
		case NUMBER:
			if((i = rgetnum(vrname(n))) >= 0) nvalue(n) = i;
			break;
		case CHAR:
			r = rgetstr(vrname(n), &bp);
			if(r) cvalue(n) =  *r;
			break;
		case BOOL:
			if(rgetflag(vrname(n))) bvalue(n) = YES;
			break;
		default:;
		};
	
	for (n = 0; nbcap[n].cap != NULL; n++)
		if(rgetflag(nbcap[n].cap)) bvalue(nbcap[n].id) = NO;
	
	if(rdata){
		char dbuf[STRSIZ];
		for(n = 0; rdata[n]; n++)
			if(*rdata[n] == '@'){
				r = ++rdata[n];
				if(rget(r,buf))
					sprintf(dbuf,"%s=%s",r,buf);
				else
					sprintf(dbuf,"%s=OFF",r);
				rdata[n] = makstr(dbuf);
			}
	}
				
	if (DV == NOSTR) {
		fprintf(stderr, "%s: missing device spec\n", host);
		exit(3);
	}
	
	DU = rgetflag("du");
	
	/* setting CU that is used in acu.c -Kenar- */
	if (DU){
		if(!PN){
			fprintf(stderr, "%s: missing phone number\n", host);
			exit(3);
		}

	   /*
		* getting real phone number if pn=@
		* This action was in acu.c in tip and I moved here,
		* and made compact it.
		* -Kenar-
		*/
		
		if(equal(PN,"@")){
			FILE *fd;
			char buf[STRSIZ];
			char *cp,*pn;
			
			if ((fd = fopen(svalue(PHONES), "r")) == NULL) {
				printf("%s: not found", svalue(PHONES));
				exit(3);
			}
			
			pn = NULL;
			while (fgets(buf, sizeof(buf), fd) != NOSTR) {
				if(index(" \t\n",*buf)) continue;
				cp = skipto(buf," \t\n");
				*cp++ = '\0';
				if (!equal(buf, host)) continue;
				pn = skip(cp," \t");
				if (*pn == '\n') {
					printf("No phone number for %s\n",host);
					fclose(fd);
					exit(3);
				}
				cp = skipto(pn," \t\n");
				*cp = '\0';
				break;
			}
			if(pn) PN = makstr(pn);
			else {
				printf("No entry for %s in %s\n",host,svalue(PHONES));
				exit(3);
			}
			/* PN=="052-937-8007,052-809-1234" for example */
		}
	}

	/* tip comment:
	 * This effectively eliminates the "hw" attribute
	 *   from the description file
	 *		if (!HW)
	 *			HW = ((CU == NOSTR) || (DU && equal(DV, CU)));
	 * 
	 * Kenar says,
	 *   If "cu" is not defined in remote, then "du" determines
	 *   whether the device is hardwired(HW) or not.
	 *   I don't know the meaning of "cu" in remote.
	 *   Limit ourself to only hayes modem,
	 *   then CU is not used anywhere in tip internal code,
	 *   execpt HW determination.
	 *   I like to make things simple: HW = !DU
	 * -Kenar-
	 */
	r = svalue(SYSSCRIPT);
	if(*r && *r != '/') {
		char buf[PATHSIZ];
		sprintf(buf,SLIPFMT,r);
		svalue(SYSSCRIPT) = makstr(buf);
	}

	svalue(HOST) = makstr(host);
	
}
