#define PATCHLEVEL 0

/*
 * Copyright (c) 1983 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
 
/*
 * Modifications by Warren Tucker <wht%n4hgf.uucp@emory.mathcs.emory.edu>
 * for eight-bit transparent pass through and file transfer protocols
 * are unencumbered in any way. They are yours, ours, everybody's, nobody's.
 */
/*+:EDITS:*/
/*:05-08-1990-15:05-wht@n4hgf-rawthru */
/*:05-08-1990-15:00-wht@n4hgf-move tip.c original to tip-orig.c */

/*
 * Copyright (c) 1995  Kenji Arisawa. All right reserved.
 * License to freely use and distribute this software is hereby granted 
 * by the author, subject to the condition that this copyright notice 
 * remains intact.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY WARRANTIES.
 */
 
/*
 *  Modifications by Kenar(Kenji Arisawa)
 *	--- E-mail: arisawa@aichi-u.ac.jp
 *
 *  1993/09 (tip+ ver.1.00)
 *  1. Enrichment of login script.
 *  2. Japanese support.
 *
 *  1993/09 (tip+ ver.101)
 *  1. Bug fix in sort routine(login.c).
 *
 *  1993/10 (tip+ ver.102)
 *  1. cdelay, ldelay support
 *  2. linesync now available
 *  3. Bug fix in receiving JIS.
 *
 *	1995/12 (tip+ ver.2.0)
 *	1. Bug fix in login.c
 *	2. tip variable xfer
 *	3. tip variable keymap
 *	4. High speed modem support
 *	5. Renewed old codes in libacu and xfer.
 *
 *  1995/12 (tip+ ver.2.1)
 *  Reconstruction of previous dirty codes.
 *  CPU dependency are now removed. And...
 *  1. Changed the format and name of script file:
 *     %begin -> begin, %end -> end, and *.tipp -> *.tip+
 *  2. Changed the name of tip variable: loginscrip -> sysscript
 *  3. Removed tip variable halfduplex
 *  4. "[set] ?" displays only WRITE permitted variables.
 *  5. "[cd]" do nothing, and "[cd] ~" changes to HOME
 *  6. Shell meta-chars are abolished in file name input.
 *  7. Log file /usr/adm/aculog is displayed by "[set] all" command,
 *     and the log format is changed from tipx's to tip's.
 *  8. Bug fix: permission problem of user script in setuid execution.
 *  9. Environment variables REMOTE and PHONES are made ineffective
 *     in setuid execution.
 *     /etc/remote and /etc/phones are now ineffective in user execution.
 *     Env.var. HOST is made ignored.
 * 10. tip+2.1 now admits various combinations of,
 *     1. remote: /etc/remote or REMOTE  ( must be defined )
 *     2. phones: /etc/phones and PHONES
 *     3. system script for tip+ given 'ss=' in remote.
 *     4. ~/.tiprc
 *     5. user script in TIPP
 * 11. Guard against device lock when sending nothing to modem.
 * 12. Bus error and segmenation violation now disconnect dial line.
 * 13. Security holes in exec system call are removed.
 * 14. Tip+ now does not change to uucp.
 * 15. Bug fix in sending file in asynchronous mode ( ~> ).
 * 16. Line delay in executing tip+ script was set 20msec.
 *     Now it is given by tip variable 'linedelay'
 *     and the default value is 200msec.
 * 17. Now user can escape tip+ script by ^C.
 *     This function is available only if 'KEYINTR' is defined in Makefile.
 * 18. Bug fix: sending file in rawftp mode.
 *
 *  1996/02 (tip+ ver.3.0)
 *  So many changes. I can't write all. The important changes are:
 *  1. Excluded dialers embedded in tip body.
 *     In stead, introduced dialers of tip+ script.
 *  2. Changed the method of text file transmission/transfer.
 *  3. Introduced user defined FTP. Tip variables rtake, ltake, rput, lput.
 *  4. Now we have a principle, one OS/CPU one binary. 
 *  5. Page and slip are cut off. They are supported on other method.
 *  6. Discarded tip variables:
 *       prompt dtrhup verbose echocheck rawftp sa da sm st rt
 *  7. Introduced new variables:
 *       rput lput rtake ltake breaksize tabsize xcontrol
 */

#include "tip.h"

char *copyright[] = {
"@(#)--------------------------------------------------------------",
"@(#)tip+ 3.0.5: tipx(x1.21.01) mod by Kenar",
"@(#)tipx (x1.21.01): tip 4.3 mod by wht@n4hgf",
"@(#)tip 4.3: (c) 1983 The Regents of the University of California.",
"@(#)------------------ All rights reserved.-----------------------",
 NULL
 };

#ifndef lint
static char sccsid[] = "@(#)tip.c 1.0 (Berkeley/Tucker/Kenar) 1/3/96";
#endif /* not lint */

static sigfunc_t sigquit();
static sigfunc_t sighup();
static sigfunc_t sigpipe();
static sigfunc_t sigbus();
static sigfunc_t sigsegv();
static sigfunc_t sigint();
static sigfunc_t cleanup();
static void user_uid();
static void daemon_uid();
static void csend(char c);


int	fildes[2];		/* file transfer synchronization channel */
int	repdes[2];		/* read process sychronization channel */
uid_t uid, euid;	/* real and effective user id's */
gid_t gid, egid;	/* real and effective group id's */
char ccc;			/* synchronization character */
int	vflag;			/* print .tiprc initialization sequence */
int rawthru = 0; 	/* raw through flag (tipx) */
int	pid;			/* pid of tipout */
int suidmode = NO;		/* Is set uid mode? (tip+) */
char **rdata = NULL; 

static char PNbuf[256];			/* This limits the size of a number */
static char *qmsg = NULL; /* quit message */

extern boolean rawflag; /* YES if in rawmode */
extern boolean conflag; /* defined in acu.c */
extern boolean lockflag; /* defined in hunt.c */

/*
 * Kenar says,
 * I discarded cu ( that may be supported by shell script )
 * and introduced new usage below.
 */
void usage(tip)
char *tip; /* argv[0] */
{
	fprintf(stderr,
		"tip usage: %s [-v] [-b speed] host\n",tip);
	fprintf(stderr,
		"tip usage: %s [-v] [-speed] host\n",tip);
	fprintf(stderr,
		"rip usage: %s -r [-v] [-b speed] host program arg1 arg2 ..\n",tip);
	fprintf(stderr,
		"rip usage: %s -r [-v] [-speed] host program arg1 arg2 ..\n",tip);
	puts("Please select speed among:");
	showbauds();
	puts("However your computer/modem may not supports all of them.");
	puts("NOTE: '-speed' is the traditional style of tip.");
	puts("      '-b speed' is the new style of tip+.");	exit(1);
}


main(argc,argv)
char *argv[];
{
	char *system = NOSTR;
	int i,ch;
	char *p;
	char sbuf[12];
	boolean rflag = 0; /* remote interface */
	int baudopt = -1;
	int optc = 0; /* count page, slip, and xopt */
	int errcnt = 0;
	char *tipname;
	extern int optind;
	extern char *optarg;

	uid = getuid();
	gid = getgid();
	euid = geteuid();	/* uucp or root */
	egid = getegid();
	suidmode = (uid != euid);
	puts( suidmode? "[uucp mode]" : "[user mode]");

	tipname = rindex(argv[0],'/');
	if(!tipname) tipname = argv[0];
	else tipname++;
	if(argc < 2) usage(tipname);

	while ((ch = getopt(argc,argv,"b:hrv0123456789")) != EOF){
		switch (ch){
		case 'h':
			usage(tipname);
		case 'r':
			rflag = YES;
			break;
		case 'v':
			vflag = YES;
			break;
		case 'b':
			baudopt = atoi(optarg);
			break;
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			/*
			 * If option is " -v -123456789" then
			 * argv[optind] is "-123456789".
			 * therefor we skip '-'.
			 * Note that 'baudopt' should be evaluated only once.
			 * -Kenar-
			 */
			if(!baudopt) {
				p = argv[optind];
				baudopt = atoi(++p);
			}
			break;
		default:
			usage(tipname);
		}
	}
	i = optind;
	if(i < argc) system = argv[i++];
	if(i < argc) rdata = &argv[i];

	if(baudopt == -1) baudopt = 0; /* use default value */
	else if(baudopt == 0){
		/* print error message and show available speed */
		errcnt++;
	}
	
	if(!system){
        perror("No host-name");
		errcnt++;
    }
	if(rflag && !rdata){
		perror("Program not specified\n");
		errcnt++;
	}

	/* checking error in options */
	if( optc > 1 ) {
		puts("Incompatible options");
		errcnt++;
	}
	
	if(errcnt) usage(tipname);
	
	if(!isatty(0) && !rflag)
	{
		fprintf(stderr,"tip: warning: input is not a tty\n");
	}

	if(!isalpha(*system)){
	   /*
		* System name is really a phone number...
		* Copy the number then stomp on the original (in case the number
		*	is private, we don't want 'ps' or 'w' to find it).
		*/
		if(strlen(system) > sizeof(PNbuf) - 1)
		{
			fprintf(stderr,"tip: phone number too long (max = %d bytes)\n",
				sizeof(PNbuf) - 1);
			exit(1);
		}
		strncpy( PNbuf,system,sizeof(PNbuf) - 1 );
		for(p = system; *p; p++) *p = '\0';
		PN = PNbuf;
		(void)sprintf(sbuf,"tip%d",nvalue(BAUDRATE));
		system = sbuf;
	}

	/* DEBUG: we examine here basic parameters
	{	int i;
		printf("system=%s\n",system);
		printf("baudopt=%d\n",baudopt);
		printf("rdata:");
		if(rdata) for(i = 0; rdata[i]; i++) printf(" %s",rdata[i]);
		puts("");
		printf("phone=%s\n",PN);
		printf("(vflag,rflag)=(%d,%d)\n",vflag,rflag); 
		exit(0);
	} */

	/* All tip+ family processes dies by SIGTERM */
	signal(SIGTERM,cleanup);
	signal(SIGINT,sigint);
	signal(SIGQUIT,sigquit);
	signal(SIGHUP,sighup);
	signal(SIGPIPE,sigpipe);
	signal(SIGBUS,sigbus);
	signal(SIGSEGV,sigsegv);
	

	/*
	 * Read environment variables before reading /etc/remote or REMOTE.
	 * -Kenar-
	 */
	for(i = 0; vname(i) != NULL; i++)
		if(vabbre(i) && isupper(*vabbre(i)) /* upper case abbre. is env. */
			&& (!suidmode || (suidmode && !(vaccess(i)&SECURE)))
			&& (p = getenv(vabbre(i))) != NULL)
			svalue(i) = p;

			
 	/*
	 * hunt: Find host entry hunting "/etc/remote" or REMOTE
	 * and initializes tip variables defined there.
	 * Hunt then opens the device and returns opened FD.
	 * ( FD is not set internally )
	 * -Kenar-
	 */
	if((FD = hunt(system,baudopt)) == ERR){
		printf("all ports busy\n");
		exit(3);
	}
	FD0 = FD1 = FD;
	
	fprintf(stderr,"[Logging in %s]\n",svalue(HOST));


	/* lconnect:
	 * connect to a device
	 * in case du,pn,at in remote: dial up to the host
	 * in case hard wired: send greeting message to the host
	 * log is taken in this process
	 * lconnect must be executed in daemon mode to get correct user.
	 * -Kenar-
	 */
	if((p = lconnect()) != NULL)
	{
		printf("\07%s\n[EOT]\n",p);
		uu_unlock();
		exit(1);
	}
	
	
	/*-------- setbuf old position------*/

	/*
	 * SYSSCRIPT is a system script for slip support.
	 * -Kenar-
	 */

	if(vdef(SYSSCRIPT)){
		if(p = login(svalue(SYSSCRIPT))){
			if(equal(p,""))
				printf("ERROR: slip script %s not found",svalue(SYSSCRIPT));
			disconnect(svalue(HOST),p);
			printf("\07%s\n[EOT]\n",p);
			uu_unlock();
			exit(1);
		}
	}


	/*
	 * Now that we have the logfile and the ACU open
	 *  return to the real uid and gid.  These things will
	 *  be closed on exit.  Swap real and effective uid's
	 *  so we can get the original permissions back
	 *  for removing the uucp lock.
	 */

	user_uid();	/* in the case of slip, we are now priviliged */

	/*  ---------------------------------------------------------
	 *                      W A R N I N G 
	 *  ---------------------------------------------------------
	 *  user_uid() executes setreuid(euid,uid) in it.
	 *  Then the process behaves as if it belongs to the user.
	 *  However NOTE that fork'ed and exec'ed processes inherit
	 *  both real and effective uids, so that these processes
	 *  have potentially a strong power.
	 *  It can be 'root' if it is executed in setuid mode of 'root'!
	 *  -Kenar-
	 *
	 *  Kenar says,
	 *    It will be the best for security to execute
	 *    setreuid(uid,uid)
	 *    here.
	 *    This action sweeps the potentiality of euid perfectly
	 *    from our process.
	 *    However we must return to euid to remove the locked file.
	 *    So we cannot execute setreuid(uid,uid) here.
	 *    
	 *    I would restrict user_uid() only here.
	 *    This is a good way for security.
	 *
	 *    Every exec process must be cleaned by
	 *    shell_uid()   i.e.  setreuid(uid,uid)
	 *    prior to execution.
	 *    It is a good manner to write down shell_uid() and execl(..)
	 *    in the same line, like that:
	 *        shell_uid(); execl(.......);
	 *    to detect dangerous mistake.
	 */


	/*
	 * tip+, tipx & tip messages
	 */
	{	int i;
		for(i = 0; copyright[i]; i++) puts(copyright[i] + 4);
	}
		

	/*
	 * vinitialize:
	 * init tip variables using ".tiprc"
	 * -Kenar-
	 */
	{	char file[256];
	 	FILE *f;
		strcpy(file, svalue(HOME));
		strcat(file, "/.tiprc");
		if ((f = fopen(file, "r")) != NULL)
		{	vinitialize(f,NULL);
			fclose(f);
			vcheck();
		}
	}
	
	/*
	 * login:
	 * init tip variables using TIPP and then login
	 * parity and pio are set in this process
	 * -Kenar-
	 */
	{	char *TIPPDIR,*msg;
		TIPPDIR = getenv("TIPP");
		if(TIPPDIR)
		{	char buf[BUFSIZ];
			sprintf(buf,"%s/%s.tip+",TIPPDIR,system);
			msg = login(buf);
			if(msg && !equal(msg,"")) kill(0,SIGTERM);
		}
	}
	
	if(rflag)
	{	/* Be careful with security problem */
		int p,status;
		pid = fork();
		if(!pid){ /* child */
			extern char **environ;
			char compath[PATHSIZ],*dirpath,path[PATHSIZ],*next;
			/* DEBUG: for(i = 0; rdata[i]; i++) puts(rdata[i]); */
			shell_uid();
			signal(SIGINT, SIG_DFL);
			dup2(FD, 0);
			dup2(FD, 1);
			for (i = 3; i < 20; i++) close(i);
			strcpy(path,getenv("PATH"));
			for(dirpath = path; dirpath; dirpath = next){
				next = index(dirpath,':');
				if(next) *next++ = '\0';
				strcpy(compath,dirpath);
				strcat(compath,"/");
				strcat(compath,rdata[0]); /* rdata[0]: command name */
				/* DEBUG: printf("command=%s\n",compath); */
				execve(compath,rdata,environ);
				/* Only if erorr, controll reach here. */
			}
			perror(compath);
			exit(1);
		}
		tio_cbreak(YES);
		while ((p = wait(&status)) > 0 && p != pid);
		tio_cbreak(NO);
		daemon_uid();
		uu_unlock();
		exit(0);
	}

	setbuf(stdin,NULL);
	
	tio_raw();

	/*
	 * Everything's set up now:
	 *	connection established (hardwired or dialup)
	 *	line conditioned (baud rate, mode, etc.)
	 *	internal data structures (variables)
	 * so, fork one process for local side and one for remote.
	 */

	printf("line speed = %d, tandem = %s\r\n",
		nvalue(BAUDRATE),
		bvalue(TAND) ? "yes" : "no");

	pipe(fildes);
	pipe(repdes);

	if(pid = fork())
		tipin();
	else
		tipout();
	/*NOTREACHED*/
}

static sigfunc_t
cleanup()
{
	daemon_uid();
	if(conflag) disconnect(svalue(HOST),qmsg);
	printf("\07%s\r\n[EOT]\r\n", (qmsg ? qmsg : ""));
	if(rawflag)	tio_unraw();
	if(lockflag) uu_unlock();
	exit(0);
}

static sigfunc_t
sigint()
{	qmsg = "User interrupt";
	kill(0,SIGTERM);
}

static sigfunc_t
sigquit()
{	qmsg = "Sig. quit";
	kill(0,SIGTERM);
}

static sigfunc_t
sighup()
{	qmsg = "Sig. hungup";
	kill(0,SIGTERM);
}

static sigfunc_t
sigpipe()
{	qmsg = "Sig. pipe";
	kill(0,SIGTERM);
}

static sigfunc_t
sigbus()
{	qmsg = "Bus error";
	kill(0,SIGTERM);
}

static sigfunc_t
sigsegv()
{	qmsg = "Seg. violation";
	kill(0,SIGTERM);
}



/*
 * Muck with user ID's.  We are setuid to the owner of the lock
 * directory when we start.  user_uid() reverses real and effective
 * ID's after startup, to run with the user's permissions.
 * daemon_uid() switches back to the privileged uid for unlocking.
 * Finally, to avoid running a shell with the wrong real uid,
 * shell_uid() sets real and effective uid's to the user's real ID.
 */

/*
 * shell_uid: switch perfectly to user's process
 * shell_uid must (and only) be executed prior to exec system call
 * like bellow.
 * shell_uid(); execl(....);
 * -Kenar-
 */
void shell_uid()
{
	setregid(gid,gid);
	setreuid(uid,uid);
}

/*
 * user_uid: switch to pseudo-user process.
 * Pseudo-user process looks like user process, but really it is not.
 * It has a power to be daemon.
 * -Kenar-
 */
static void user_uid()
{
	setregid(egid,gid);
	setreuid(euid,uid);
}

/*
 * daemon_uid: switch to daemon process.
 * This function makes effect only in pseudo-user process.
 * -Kenar-
 */
static void daemon_uid()
{
	setreuid(uid,euid);
	setregid(gid,egid);
}



/*
 * ****TIPIN   TIPIN****
 */
/*
 *	tipin() read stdin and send the codes to modem.
 *	- Kenar -
 */
tipin()
{
	char gch;
	boolean bol;
	int raise; /* current raise status */
	int col; /* column position to expand tab */
		
	/*
	 * Kinda klugey here...
	 *   check for scripting being turned on from the .tiprc file,
	 *   but be careful about just using setscript(), as we may
	 *   send a SIGEMT before tipout has a chance to set up catching
	 *   it; so wait a second, then setscript()
	 *
	 * Kenar says,
	 *	 Tipx func setscript() is replaced by sigEMT(SCRIPT).
	 *	 see: cmds.c
	 */
	
	if(bvalue(SCRIPT)){
		sleep(1);
		sigEMT(SCRIPT);
	}
	
	raise = bvalue(RAISE);
	col = 0;
	bol = YES;

	while(1){
		gch = getchar();
		/* ESCAPE should be the first to avoid masking of cvalue(ESCAPE) */
		if(gch == cvalue(ESCAPE) && bol){
				if(!(gch = escape())) continue;
		}
		bol = NO;
		if(!rawthru){
			if(bvalue(RAISE)
				&& gch == cvalue(RAISECHAR)){
				raise = !raise;
				continue;
			}
			if(gch == cvalue(FORCE)) gch = getchar();
			else if(raise && islower(gch)) gch = toupper(gch);
			else if(vdef(KEYMAP) && gch == svalue(KEYMAP)[0])
				gch = svalue(KEYMAP)[2];
			bol = any(gch,svalue(EOL));

			if (gch == '\t' && bvalue(TABEXPAND)) {
				csend(' ');
				while ((++col % nvalue(TABSIZE)) != 0) csend(' ');
				continue;
			}
		}
		
		csend(gch);
		col++;
		if(gch == '\r'){
			bol = YES;
			col = 0;
		}
	}
}

static void csend(c)
	char c;
{
	pwrite(&c, 1);
	if(bvalue(LOCALECHO)) putchar(c);
	if(c == '\r'){
		if (nvalue(LDELAY) > 0) nap(nvalue(LDELAY));
		if(!rawthru && bvalue(LOCALECHO)) putchar('\n');
	}
	else if(nvalue(CDELAY) > 0) nap(nvalue(CDELAY));
}


any(c,p)
register char c,*p;
{
	while(p && *p)
		if(*p++ == c)
			return(1);
	return(0);
}

size(s)
register char *s;
{
	register int i = 0;

	while(s && *s++)
		i++;
	return(i);
}


char *
ctrl(c)
char c;
{
	static char s[3];

	if(c < 040 || c == 0177)
	{
		s[0] = '^';
		s[1] = c == 0177 ? '?' : c+'A'-1;
		s[2] = '\0';
	}
	else 
	{
		s[0] = c;
		s[1] = '\0';
	}
	return(s);
}

