/*
 * Copyright (c) 1983 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	@(#)tip.h	5.4 (Berkeley) 9/2/88
 */

/*
 * Copyright (c) 1995  Kenji Arisawa. All right reserved.
 * License to freely use and distribute this software is hereby granted 
 * by the author, subject to the condition that this copyright notice 
 * remains intact.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY WARRANTIES.
 */
 

/*
 * tip - terminal interface program
 */

#include "config.h"

#undef CTRL
#define	CTRL(c)	((c)&037)


/*
 * Kenar says,
 *   It may be too late to typedef boolean to use consistently.
 */
typedef int boolean;


/*
 * Remote host attributes
 */
char *DV;			/* UNIX device(s) to open */
char *CM;			/* initial connection message */
char *AT;			/* acu type */
char *PN;			/* phone number(s) */
boolean DU;			/* this host is dialed up */

/*
 * Definition of indices into variable table so
 *  value(DEFINE) turns into a static address.
 */

#define BEAUTIFY	0
#define BAUDRATE	1
#define DIALTIMEOUT	2
#define EOFREAD		3
#define EOFWRITE	4
#define EOL		5
#define ESCAPE		6
#define EXCEPTIONS	7
#define FORCE		8
#define FRAMESIZE	9
#define HOST		10
#define LOG		11
#define PHONES		12
#define UNUSED2		13
#define RAISE		14
#define RAISECHAR	15
#define RECORD		16
#define REMOTE		17
#define SCRIPT		18
#define TABEXPAND	19
#define TABSIZE		20
#define SHELL		21
#define HOME		22
#define	LOCALECHO	23
#define DISCONNECT	24
#define TAND		25
#define LDELAY		26
#define CDELAY		27
#define RTAKE		28
#define LTAKE		29
#define RPUT		30
#define	LPUT		31
#define	PARITY		32
#define LINESYNC    33
/* added by Kenar */
#define SYSSCRIPT	34
#define BRKSIZ		35
#define LANGTYPE	36
#define XFER		37
#define KEYMAP		38
#define XCONTROL	39



/*
 * String value table
 */
typedef
	struct {
		char	*v_name;	/* whose name is it */
		char	v_type;		/* for interpreting set's */
		char	v_access;	/* protection of touchy ones */
		char	*v_abbre;	/* possible abbreviation */
		char	*v_rname;	/* abbreviation used in /etc/remote */
		void	*v_value;	/* pointer to data */
	}
	value_t;

/*
 * I hate CPU dependency and so replaced original codes...
 * -Kenar-
 */

#define value(v)	(vtable[v].v_value)
#define svalue(v)	((char*)vtable[v].v_value)
#define nvalue(v)	(*(int*)vtable[v].v_value)
#define bvalue(v)	(*(boolean*)vtable[v].v_value)
#define cvalue(v)	(*(char*)vtable[v].v_value)

#define vname(v)	vtable[v].v_name
#define vtype(v)	vtable[v].v_type
#define vaccess(v)	vtable[v].v_access
#define vabbre(v)	vtable[v].v_abbre
#define vrname(v)	vtable[v].v_rname

/* Use vdef/vndef to detect the string value is defined or not */
#define vdef(v)		(*(char*)vtable[v].v_value != '\0')
#define vndef(v)	(*(char*)vtable[v].v_value == '\0')
/* Use vundef to eliminate the definition of the string value */
#define vundef(v)	(char*)vtable[v].v_value = ""

#define STRING	01		/* string valued */
#define BOOL	02		/* true-false value */
#define NUMBER	04		/* numeric value */
#define CHAR	010		/* character value */
#define TMASK	017		/* type mask */

#define	READ	01		/* read access */
#define WRITE	02		/* write access to variable */
#define SECURE	04		/* Inhibit the use of environment in daemon exec. */
#define CHANGED	010		/* low bit is used to show modification */


#define	equal(a, b)	(strcmp(a,b)==0)/* A nice function to string compare */


/*
 * Escape command table definitions --
 *   lookup in this table is performed when ``escapec'' is recognized
 *   at the begining of a line (as defined by the eolmarks variable).
*/

typedef
	struct {
		char	e_char;		/* char to match on */
		char	*e_help;	/* help string */
		void 	(*e_func)();	/* command */
	}
	esctable_t;

boolean	vflag;		/* verbose during reading of .tiprc file */
extern value_t	vtable[];	/* variable table */

#define NOSTR	((char *)NULL)
#define NOFILE	((FILE *)NULL)
#define NOPWD	((struct passwd *)0)


int	FD;			/* open file descriptor to remote host */
int FD0;
int FD1;

void abort_slip(void);
char *ctime();
char *ctrl(char c);
void disconnect(char *host,char *reason);
int escape(void);
char *expand(char *s);
char *getenv();
struct passwd *getpwuid();
int hunt(char *host, int baudopt);
char *lconnect();
void logent(char *host, char *num, char *acu, char *message);
char *login(char *path);
void loginit(char *logfile);
void nap(int ms);
void shell_uid(void);
void show(char *cp, int n);
int size(char *s);
long time();
int uu_lock(char *devname);
void uu_unlock(void);
int vinitialize(FILE *fp,char *term);
void vlex(char *s);
void vcheck(void);



/*
 * Addition by Kenar
 */
#define PATHSIZ 256 /* maximum path size we support */
#define STRSIZ  256 /* maximum string size to display */

/* Child control through signal */
#define CWAIT "CWAIT"    /* wait */
#define CSYNC  "CSYNC"   
#define CRTHON "CRTHON"  /* raw through on */
#define CRTHOFF "CRTHOFF" /* raw through off */
#define CSWEEPON "CSWEEP:1" /* sweep out received chars */
#define CSWEEPOFF "CSWEEP:0" /* receive normally */

#define makstr(x) ((char*)strcpy((char*)malloc(strlen(x)+1),x))
#define lett unsigned char

/* Pointers to com. port I/O function introduced by Kenar */
/* They are defined by setpio() in pio.c  */
#define pread(buf,size)  (*cread)(buf,size)
#define pwrite(buf,n)	(*cwrite)(buf,n)
int (*cread)();
int (*cwrite)();


/* ioctl functions (ioctl.c) -Kenar- */
void tio_raw(void);
void tio_unraw(void);
void tio_cbreak(boolean mode);
void tio_tandem(boolean mode);
void ttysetup(int fd,int baudcode,boolean tandmode);
void clrdtr(int fd);
void setdtr(int fd);
void iflush(int fd);
int nread(int fd);
void setxlock(int fd);
void sethup(int fd);
void sendbrk(int fd,int msec);
void io_tandem(int fd,int cntl);

/* defined in speed.c */
int baudcode(int n);
void showbauds(void);


/* Frequently used function of Kenar (misc.c) */
char *strapp(char*,char*);
char *skip(char *s, char *str);
char *skipto(char *s, char *str);
char *arg0(char *s);
char *trim(char *bp);
int oneof(char *s,char *r);
char *getoken(char *bp,char *sep);
char *strlower(char*);
char *strupper(char*);
int sepstr(char* str,char** argp,char* sep);
boolean inmem(char c,char *bp,int siz);
int chcount(char c,char *bp,int siz);
char *jdate(void);

/* I dislike 'do{..}while()' syntax of C, so I define: -Kenar- */
#define repeat do
#define until(x) while(!(x))

#define ERR (-1)
#define YES 1
#define NO 0

