/*
 * Copyright (c) 1983 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
/*+:EDITS:*/
/*:05-08-1990-15:29-wht@n4hgf-rawthru */
/*:05-08-1990-15:28-wht@n4hgf-mv tipout.c tipout-orig.c */

#ifndef lint
static char sccsid[] = "@(#)tipout.c	5.2 (Berkeley) 9/2/88";
#endif /* not lint */

#include "tip.h"
/*
 * tip
 *
 * lower fork of tip -- handles passive side
 *  reading from the remote host
 */

#define sendOK()  write(repdes[1], (char *)&ccc, 1);

extern int	fildes[2];		/* file transfer synchronization channel */
extern int	repdes[2];		/* read process sychronization channel */
extern char	ccc;			/* synchronization character */
extern int rawthru;
static boolean csweep = NO;
static boolean syncwatch = NO;
static int ppid = 0;

/*
 * 	script file descriptions
 *	fscript == NULL and tiprecord = NULL must be equivalent
 *  These variables should be packed to struct type in future.
 */
static FILE	*fscript = NULL; /* FILE for scripting */
static int rcflush = 0; /* No record flush */
static char *tiprecord = NULL; /* "$(HOME)/.tiprecord" while scripting */



/*
 * TIPOUT wait state routine --
 *   sent by TIPIN when it wants to posses the remote host
 */
sigfunc_t
intIOT()
{
	char line[256];
	int n;

	/* We should block signals */
	n = 0;
	do{
		read(fildes[0], &line[n], 1);
	} while(line[n++]);
	
	if(equal(line,CWAIT)){
		write(repdes[1],&ccc,1);
		read(fildes[0], &ccc,1);
	}
	else if(equal(line,CRTHON)) rawthru = 1;
	else if(equal(line,CRTHOFF)) rawthru = 0;
	else if(equal(line,CSWEEPON)) csweep = 1;
	else if(equal(line,CSWEEPOFF)){
		csweep = 0;
		syncwatch = 0;
	}
}


FILE *openrec()
/*
 *	return: open fd for fscript
 *	side effect: rcflush, tiprecord
 */
{	int n;
	char buf[STRSIZ];
	FILE *fp = NULL;
	if(vndef(RECORD))
		puts("record file is not specified\r");
	else {
		n = sscanf(svalue(RECORD),"%s %d",buf,&rcflush);
		fp = fopen(buf, "a");
		if(!fp) printf("can't create %s\r\n", buf);
		else fchmod(fileno(fp),0600);
		if(n < 2) rcflush = 0;
		//printf("record=%s\nrcflush=%d\n",buf,rcflush);
		if(fp){ /* registrate the name to ~/.tiprecord */
			FILE *fp;
			char rcname[STRSIZ];
			strcpy(rcname,svalue(HOME));
			fp = fopen(strcat(rcname,"/.tiprecord"),"w");
			fputs(buf,fp);
			fclose(fp);
			tiprecord = makstr(rcname);
		}
	}
	return fp;
}


/*
 *  intEMT:
 *  Accepts changes of tip variables over the pipe and acts accordingly.
 *	see also sigEMT in cmds.c
 *
 *  Kenar says,
 *    All changes of tip variables should be accepted through intEMT.
 *  1995/12: Completely rewritten by Kenar for tip+ support.
 *	-Kenar-
 */
sigfunc_t
intEMT()
{
	char line[256],*s;
	int n,id;
	
	/* we should block signals */
	
	n = 0;
	do{
		read(fildes[0], &line[n], 1);
	} while(line[n++]);
	
	/* DEBUG 
	printf("intEMT::%s\r\n",line);
	*/
	s = index(line,':');
	*s++ = '\0';
	id = atoi(line);
	/* Don't use sscanf. STRING type data may not be printable. */
	switch(vtype(id)&TMASK){
	case STRING:
		svalue(id) = makstr(s);
		break;
	case NUMBER:
		nvalue(id) = atoi(s);
		break;
	case BOOL:
		bvalue(id) = atoi(s);
		break;
	case CHAR:
		cvalue(id) = *s;
		break;
	}
	
	switch(id){
		case RECORD:/* We should permit the change of record in scripting */
			if(fscript){
				fclose(fscript);
				fscript = NULL;	
			}
			if(bvalue(SCRIPT)) fscript = openrec();
			break;
		case SCRIPT:/* I am afraid of receiving "SCRIPT:TRUE" in scripting */
			if(bvalue(SCRIPT)){
				if(fscript) break;
				fscript = openrec();
			}
			else{
				if(fscript){
					fclose(fscript);
					fscript = NULL;
				}
				if(tiprecord){
					unlink(tiprecord);
					tiprecord = NULL;
				}
			}
			break;
		case PARITY: setpio(svalue(PARITY)); break;
		case LANGTYPE: setpio(svalue(LANGTYPE)); break;
		case TAND: tandem(bvalue(TAND)); break;
		default:;
	};
}

sigfunc_t
intTERM()
{
	if (bvalue(SCRIPT) && fscript != NULL) fclose(fscript);
	if(tiprecord) unlink(tiprecord);
	exit(0);
}

sigfunc_t
intUSR1()
{
	syncwatch = YES;
}



/* tipx */
extern int rawthru;


sigfunc_t
sigbus()
{
	kill(ppid,SIGBUS);
	sleep(5); /* To be killed by the parent */
}

sigfunc_t
sigsegv()
{
	kill(ppid,SIGSEGV);
	sleep(5); /* To be killed by the parent */
}



/*
 * ****TIPOUT   TIPOUT****
 */
/* tip+ */
#define	ALLSIGS \
sigmask(SIGEMT)|sigmask(SIGTERM)|sigmask(SIGIOT)|sigmask(SIGUSR1)

/* tipx: ----
#define	ALLSIGS	sigmask(SIGEMT)|sigmask(SIGTERM)|sigmask(SIGIOT)|sigmask(SIGUSR1)|sigmask(SIGUSR2)
*/

/* tip: ----
#define	ALLSIGS	sigmask(SIGEMT)|sigmask(SIGTERM)|sigmask(SIGIOT)|sigmask(SIGSYS)
*/

/*
 *	tipout() read chars from modem and put them to stdout.
 *	- Kenar -
 */
tipout()
{
	extern int errno;
	lett buf[BUFSIZ];
	register lett *cp;
	register int cnt;
	int omask;
	char synch;
	char xquit;
	int nlcount;

	ppid = getppid();

	signal(SIGINT, SIG_IGN);
	signal(SIGQUIT, SIG_IGN);
	signal(SIGEMT, intEMT);		/* attention from TIPIN */
	signal(SIGTERM, intTERM);	/* time to go signal */
	signal(SIGIOT, intIOT);		/* scripting going on signal */
	signal(SIGHUP, intTERM);	/* for dial-ups */
	/* tip+ */
	signal(SIGBUS,sigbus);
	signal(SIGSEGV,sigsegv);
	signal(SIGUSR1,intUSR1);

dispmode:
	/*
	* regular execution
	* scweepmode: turned on by signal
	*/
	
	nlcount = 0;
	for(;;)
	{	cnt = pread(buf, BUFSIZ);
		/*	Here all chars in port buffer are read.		 */
		/*
		 *	If buffer is empty, pread wait at least one char is received.
		 *	-Kenar -
	     */
		if (cnt <= 0) goto nocarrier;
		/* block signals while writing. -Kenar - */
		omask = sigblock(ALLSIGS);
		write(1,buf,cnt);
		if (bvalue(SCRIPT) && fscript != NULL){
			/* count '\n' in buf */
			if(rcflush)
				for (cp = buf; cp < buf + cnt; cp++)
					if(*cp == '\n') nlcount++;
			/* record them to tip.record */
			if (bvalue(BEAUTIFY)){/* except (CTRL-code) - EXCEPTIONS */
				for (cp = buf; cp < buf + cnt; cp++)
					if (!(*cp < ' ') || any(*cp,svalue(EXCEPTIONS)))
						putc(*cp, fscript);
			}
			else
				fwrite(buf, 1, cnt, fscript);
			if(rcflush && nlcount >= rcflush){
				fflush(fscript);
				nlcount = 0;
			}
		}
		sigsetmask(omask);
		if(csweep) goto sweepmode;
	};


nocarrier:
	/* if(cnt < 0 && errno == EIO) puts("Lost carrier.\r");
		else puts("Something bad!.\r"); */
	if(cnt <= 0) puts("Lost carrier.\r");
	kill(ppid,SIGTERM);
	sleep(5); /* To be killed by parent */
	/*NOT REACHED*/
	puts("Something bad!. Please report to <arisawa@aichi-u.ac.jp>\r");
	exit(0); /* Guard for FreeBSD */


sweepmode:
   /* tip+
	* sweep out steam watching '\r'
	* Done in case bvalue(LINESYNC)==YES
	* syncwatch: tunrned on by signal SIGUSR1
	* -Kenar-
	*/
	xquit = svalue(XCONTROL)[0];
	xquit = (xquit == '-' ?  0 : CTRL(xquit));
	synch = '\r';
	for(;csweep;){
		cnt = pread(buf, BUFSIZ);
		if (cnt <= 0) goto nocarrier;
		if(xquit && index(buf,xquit)) goto interrupt;
		if(syncwatch && inmem(synch,buf,cnt)){
			if(kill(ppid,SIGALRM)) printf("SIGALRM not caught");
			syncwatch = NO;
		}
	}
	goto dispmode;
	

interrupt:
	kill(ppid, SIGINT);
	csweep = NO;
	puts("Interrupt by remote host\r");
	goto dispmode;

}
