/*
 * Copyright (c) 1983 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
 * Copyright (c) 1995  Kenji Arisawa. All right reserved.
 * License to freely use and distribute this software is hereby granted 
 * by the author, subject to the condition that this copyright notice 
 * remains intact.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY WARRANTIES.
 */
 
 
/*                   ----------------
 *                         NOTE
 *                   ----------------
 * Vtable in this file is completely reconstructed by Kenar for tip+ support.
 * Kenar made the table user friendly so that it reflect only administrator's
 * or user's needs.
 * That is, Kenar purged the table of elements
 * that come only from special technical reason in programming.
 *
 * 1995/12
 * -Kenar-
 */

#ifndef lint
static char sccsid[] = "@(#)vars.c	1.0 (Berkeley/Kenar) 1/3/96";
#endif /* not lint */

#include "tip.h"

/* ------------------------------
 * Init process of tip+ variables
 * ------------------------------
 * 1. The assignment in vtable in this file.
 * 2. The environ variable such as HOME, SHELL, REMOTE, PHONES.
 * 3. The definition in /etc/remote or REMOTE
 *    remote may define: at,br,cm,cu,di,du,dv,el,fs,hd,ie,oe,pa,pn,tc
 * 4. The definition in the system script given by "ss=" in remote file.
 * 5. The definition in ~/.tiprc
 * 6. The definition in the user script given by TIPP
 * 7. The set command by an user executing tip+
 * -Kenar-
 */



/* ------------------------
 * Attributes for variables
 * ------------------------
 * moved from "tip.h" to this file by Kenar.
 * All variables of a type NUMBER, BOOL, CHAR must be explicitly
 * initialized here.
 * That is tip+ style ( not tip type ).
 * -Kenar-
 */
static int BE=NO; /* beautify */
static int BR=DEFBR; /* beud rate */
static int DT=60;	/* dial time out */
static char ES='~';		/* escape character */
static char FO=CTRL('p');	/* force (literal next) char*/
static int	FS=DEFFS;		/* frame size for transfers */
static int	RA=NO; 		/* raise */
static char RC=CTRL('a');	/* raise character */
static int	SC=NO; /* script */
static int	TX=NO; /* tab expand */
static int	TS=8; 	/* tab size */
static int TA=YES; /* tandem */
static int LD=200;	/* line delay for file transfers to remote */
static int CD=0;	/* char delay for file transfers to remote */
static int LE=NO; /* local echo */
static int RT=NO; /* set route */
static int LI=NO; /* linesync */
static int BS=300; 	/* break length */


/* -----------------------
 * Definition of variables
 * -----------------------
 * Comments below are for tip+. Be aware that is not for tip.
 * Tip+2.1 redefined the table and changed it's meanings.
 *
 * The data structure of vtable:
 		char	*v_name;	--- whose name is it
		char	v_type;		--- for interpreting set's
		char	v_access;	--- protection of touchy ones
		char	*v_abbre;	--- possible abbreviation
		char	*v_rname;	--- abbreviation used in /etc/remote
		void	*v_value;   --- must be address unlike tip or tipx
 *
 * v_name
 *   1. This name is used in printing/setting values in [set].
 *   2. NOSTR means the termination of vtable.
 *
 * v_type
 *   Takes 4 types: STRING, NUMBER, BOOL, CHAR
 *   1. If the variable is unused this field must be 0.
 *
 * v_access
 *   A pair of keywords (READ,WRITE) and a optional keyword SECURE
 *   are provided. 
 *   1. READ is read-permission for users.
 *      The command "[set] all" will display all READ variables.
 *      READ must be assigned for all variables except unused one.
 *   2. WRITE is write-permission for uses.
 *      Variables with WRITE permission can be redefined
 *      in .tiprc, login script and [set] command.
 *      You can assign a value to all READ only variables in reading
 *      environment variables, remote file and slip script.
 *      (slip script: a file defined by "ss=" in the remote file.)
 *   3. And we have a option,
 *      SECURE: Inhibit the use of environment variable
 *              in setuid execution for security reason.
 *   4. If the variable is unused this field must be 0.
 *
 * v_abbre
 *   1. abbreviation that is useful for setting values in [set] command,
 *      .tiprc, tip+ script files.
 *   2. This name is also used initializing by environment variable.
 *      Then use the name of environment variable. It should be capitals
 *      as it is.
 *   3. If the variable is unused this field must be NOSTR.
 *
 * v_rname
 *   1. Tip variables with v_rname is initialized
 *      by /etc/remote or $REMOTE if it can be.
 *   2. If the variable is unused this field must be NOSTR.
 *
 * v_value
 *   1. v_value must be a pointer and the real data must be in *v_value.
 *   2. STRING type alway has a value including "".
 *      Never assign NOSTR/NULL instead of "".
 *      Use vundef() to assign "" to this field.
 *      Use vdef()/vndef() in coding to detect defined/undefined string value.
 *      vndef() detect "" in this field.
 *   3. CHAR type always has a value even if it's value is '\0'.
 *   4. If the variable is unused this field must be NULL.
 *
 * -Kenar-
 */
value_t vtable[] = {
	{ "beautify",	BOOL,		READ|WRITE,
	  "be",	NOSTR,	&BE },
	{ "baudrate",	NUMBER, 	READ,
	  "ba",	"br",	&BR },	/* NOTE1 */
	{ "dialtimeout",NUMBER,		READ,
	  "dial","dt",	&DT },  /* added "dt" */
	{ "eofread",	STRING, 	READ|WRITE,
	  "eofr", "ie", "\001" }, /* NOTE1 */ 	{ "eofwrite",	STRING, 	READ|WRITE,
	  "eofw", "oe", "\004" }, /* NOTE1 */ 
	{ "eol",	STRING,			READ|WRITE,
	  NOSTR,	"el",	"" },
	{ "escape",	CHAR,			READ|WRITE,
	  "es",	NOSTR,	&ES },
	{ "exceptions",	STRING, 	READ|WRITE,
	  "ex",	NOSTR,	"\t\n\b\f" },
	{ "force",	CHAR,			READ|WRITE,
	  "fo",	NOSTR,	&FO },
	{ "framesize",	NUMBER, 	READ|WRITE,
	  "fr", "fs",		&FS },
	{ "host",	STRING,			READ,
	  "ho",	NOSTR,	"" },
	{ "log",	STRING,			READ,
	  NOSTR, NOSTR,	ACULOG },  /* see config.h for ACULOG */
	{ "phones",	STRING,			READ|SECURE,
	  "PHONES", NOSTR,	"/etc/phones" },
	{ "unused2",	0,			0,
	  NOSTR,	NOSTR,	NULL },
	{ "raise",	BOOL,			READ|WRITE,
	  "ra",	NOSTR,	&RA },
	{ "raisechar",	CHAR,		READ|WRITE,
	  "rc",	NOSTR,	&RC },
	{ "record",	STRING,			READ|WRITE,
	  "re",	NOSTR, "tip.record"},
	{ "remote",	STRING,			READ|SECURE,
	  "REMOTE", NOSTR,	"/etc/remote" },
	{ "script",	BOOL,			READ|WRITE,
	  "sc",	NOSTR,	&SC },
	{ "tabexpand",	BOOL,		READ|WRITE,
	  "tx", NOSTR,	&TX },
	{ "tabsize",	NUMBER,		READ|WRITE,
	  "ts", NOSTR,	&TS },
	{ "SHELL",	STRING,			READ|WRITE,
	  "SHELL",	NOSTR,	"/bin/sh" },
	{ "HOME",	STRING,			READ,
	  "HOME", NOSTR,	"" },
	{ "localecho",	BOOL,		READ|WRITE,
	  "le",	"hd",	&LE },
	{ "disconnect",	STRING, 	READ|WRITE,
	  "di",	"di",	"" },
	{ "tandem",	BOOL,			READ|WRITE,
	  "ta", NOSTR,		&TA },
	{ "linedelay",	NUMBER, 	READ|WRITE,
	  "ldelay",	NOSTR,	&LD },
	{ "chardelay",	NUMBER, 	READ|WRITE,
	  "cdelay", NOSTR,	&CD },
	{ "rtake",	STRING, 		READ|WRITE,
	  NOSTR,	NOSTR,	"cat ?;cat>/dev/null" },
	{ "ltake",	STRING,			READ|WRITE,
	  NOSTR, NOSTR,	"" },
	{ "rput",	STRING,			READ|WRITE,
	  NOSTR, NOSTR,	"cat>?" },
	{ "lput",	STRING,			READ|WRITE,
	  NOSTR, NOSTR,	"" },
	{ "parity",	STRING,			READ|WRITE,
	  "par", "pa",	"none" },
	{ "linesync",   BOOL,		READ|WRITE,
	  "li",	NOSTR,	&LI },
	{ "sysscript",  STRING,		READ, 
	  "ss",	"ss",	"" },
	/* tip+ extension by Kenar */
	{ "breaksize",	NUMBER,	READ|WRITE,
	  "bs",	"bs",	&BS }, 
	{ "language",	STRING,		READ|WRITE,
	  "lan", NOSTR,	"ascii" },	/* japanese kanji code */
	{ "xfer",	STRING,			READ|WRITE,
	  "xf",	NOSTR, "" },		/* file xfer protocol */
	{ "keymap",	STRING,			READ|WRITE,
	  "km",	NOSTR, "" },		/* keymap */
	{ "xcontrol",	STRING, 	READ|WRITE,
	  "xc", NOSTR, "---C" },
	{ NOSTR, 0, 0, NOSTR, NOSTR, NULL }
};

/* ==========================================================
NOTE1
The abbreviation in /etc/remote is
"br" for baudrate or "ba"
"fs" for framesize or "fr"
"ie" for eofread or "eofr"
"oe" for eofwrite or "eofw"
and they are usially defined such like
ie=#$%:oe=^D
*/
